import {MeterbusLibExceptions} from './exceptions'
import {MeterbusLibFrames} from './simpleframes'
import {MeterbusLibUtils} from './utils'
import {MeterbusLibCodeTables} from './codetables'

export namespace MeterbusLibLongFrame {
    export class LongFrame extends MeterbusLibFrames.ControlFrame {
        private _fixedDataHeader : FixedDataHeader
        get fixedDataHeader() : FixedDataHeader { return this._fixedDataHeader }

        constructor(telegram : Uint8Array) {
            super(telegram)
        }

        parse2() {
            super.parse2() // control frame parse2 method
            this._fixedDataHeader = new FixedDataHeader(this._telegram.slice(7, 19))
            this._fixedDataHeader.parse()
        }

        getJSON() : string {
            return JSON.stringify(this)
        }
    }

    export class FixedDataHeader {
        private _data : Uint8Array

        private _identNo : number
        private _manufacturer : string
        private _version : number
        private _medium : string
        private _accessNo : number
        private _status : number
        private _signature : Uint8Array

        get identNo() : number { return this._identNo }
        get manufacturer() : string { return this._manufacturer }
        get version() : number { return this._version }
        get medium() : string { return this._medium }
        get accessNo() : number { return this._accessNo }
        get status() : number { return this._status }
        get signature() : Uint8Array { return this._signature }

        constructor(data : Uint8Array) {
            this._data = data
        }

        parse() : void {
            this._identNo = MeterbusLibUtils.bcd(this._data.slice(0, 4))
            this._manufacturer = MeterbusLibUtils.manufCode(this._data.slice(4, 6))
            this._version = this._data[6]
            this._medium = MeterbusLibCodeTables.MEDIUM_CODE[this._data[7]]
            this._accessNo = this._data[8]
            this._status = this._data[9]
            this._signature = this._data.slice(10, 12)
        }

        getJSON() : string {
            return JSON.stringify(this)
        }
    }

}