import {MeterbusLibExceptions} from './exceptions'
import {MeterbusLibFrames} from './simpleframes'
import {MeterbusLibLongFrame} from './longframe'

export namespace MeterbusLib {


    export class Telegram {
        private _hexString : string
        private _telegram : Uint8Array
        private _frame : MeterbusLibFrames.Frame

        constructor() {
        }
    
        get hexString() : string {
            return this._hexString
        }

        get telegram() : Uint8Array {
            return this._telegram
        }

        get frame() : MeterbusLibFrames.Frame {
            return this._frame
        }

        fromHexString(hexString : string) {
            this._hexString = hexString
            this._telegram = new Uint8Array(this._hexString.split(' ').map<number>((val) => {
                let n = parseInt(val, 16)
                if ((n < 0) || (n > 255)) {
                    throw new MeterbusLibExceptions.IllegalNumberError()
                }
                return n
            }))
        }

        parse() {
            if (this._telegram[0] === 0x68 && this._telegram[1] === 0x03) {
                this._frame = new MeterbusLibFrames.ControlFrame(this._telegram)
            } else if (this._telegram[0] === 0x68) {
                this._frame = new MeterbusLibLongFrame.LongFrame(this._telegram)
            } else if (this._telegram[0] === 0x10) {
                this._frame = new MeterbusLibFrames.ShortFrame(this._telegram)
            } else if (this._telegram[0] === 0xe5) {
                this._frame = new MeterbusLibFrames.SingleCharFrame(this._telegram)
            } else {
                throw new MeterbusLibExceptions.InvalidStartCharError()
            }

            this._frame.parse()
        }
    }
}